"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_order_ import ListResourceOrder, ListResourceOrderTypedDict
from .ordersortproperty import OrderSortProperty
from .productpricetype import ProductPriceType
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, TypedDict, Union
from typing_extensions import Annotated, NotRequired


OrdersListQueryParamOrganizationIDFilterTypedDict = Union[str, List[str]]
r"""Filter by organization ID."""


OrdersListQueryParamOrganizationIDFilter = Union[str, List[str]]
r"""Filter by organization ID."""


OrdersListQueryParamProductIDFilterTypedDict = Union[str, List[str]]
r"""Filter by product ID."""


OrdersListQueryParamProductIDFilter = Union[str, List[str]]
r"""Filter by product ID."""


QueryParamProductPriceTypeFilterTypedDict = Union[
    ProductPriceType, List[ProductPriceType]
]
r"""Filter by product price type. `recurring` will return orders corresponding to subscriptions creations or renewals. `one_time` will return orders corresponding to one-time purchases."""


QueryParamProductPriceTypeFilter = Union[ProductPriceType, List[ProductPriceType]]
r"""Filter by product price type. `recurring` will return orders corresponding to subscriptions creations or renewals. `one_time` will return orders corresponding to one-time purchases."""


UserIDFilterTypedDict = Union[str, List[str]]
r"""Filter by customer's user ID."""


UserIDFilter = Union[str, List[str]]
r"""Filter by customer's user ID."""


class OrdersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[OrdersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    product_id: NotRequired[Nullable[OrdersListQueryParamProductIDFilterTypedDict]]
    r"""Filter by product ID."""
    product_price_type: NotRequired[Nullable[QueryParamProductPriceTypeFilterTypedDict]]
    r"""Filter by product price type. `recurring` will return orders corresponding to subscriptions creations or renewals. `one_time` will return orders corresponding to one-time purchases."""
    user_id: NotRequired[Nullable[UserIDFilterTypedDict]]
    r"""Filter by customer's user ID."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[OrderSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class OrdersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[OrdersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[OrdersListQueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    product_price_type: Annotated[
        OptionalNullable[QueryParamProductPriceTypeFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product price type. `recurring` will return orders corresponding to subscriptions creations or renewals. `one_time` will return orders corresponding to one-time purchases."""

    user_id: Annotated[
        OptionalNullable[UserIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer's user ID."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[OrderSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "product_id",
            "product_price_type",
            "user_id",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "organization_id",
            "product_id",
            "product_price_type",
            "user_id",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class OrdersListResponseTypedDict(TypedDict):
    result: ListResourceOrderTypedDict


class OrdersListResponse(BaseModel):
    next: Callable[[], Optional[OrdersListResponse]]

    result: ListResourceOrder



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .platforms import Platforms
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class DonationOrganizationTypedDict(TypedDict):
    id: str
    name: str
    avatar_url: str
    is_personal: bool
    platform: Platforms


class DonationOrganization(BaseModel):
    id: str

    name: str

    avatar_url: str

    is_personal: bool

    # fmt: off
    PLATFORM: Annotated[Final[Platforms], pydantic.Field(alias="platform")] = Platforms.GITHUB # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pledge_input import PledgeInput, PledgeInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookPledgeUpdatedPayloadType(str, Enum):
    PLEDGE_UPDATED = "pledge.updated"


class WebhookPledgeUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a pledge is updated.

    **Discord & Slack support:** Basic
    """

    data: PledgeInputTypedDict
    type: WebhookPledgeUpdatedPayloadType


class WebhookPledgeUpdatedPayload(BaseModel):
    r"""Sent when a pledge is updated.

    **Discord & Slack support:** Basic
    """

    data: PledgeInput

    # fmt: off
    TYPE: Annotated[Final[WebhookPledgeUpdatedPayloadType], pydantic.Field(alias="type")] = WebhookPledgeUpdatedPayloadType.PLEDGE_UPDATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metricsintervallimit import MetricsIntervalLimit, MetricsIntervalLimitTypedDict
from polar_sdk.types import BaseModel
from typing import TypedDict


class MetricsIntervalsLimitsTypedDict(TypedDict):
    r"""Date interval limits to get metrics for each interval."""

    hour: MetricsIntervalLimitTypedDict
    r"""Limits for the hour interval."""
    day: MetricsIntervalLimitTypedDict
    r"""Limits for the day interval."""
    week: MetricsIntervalLimitTypedDict
    r"""Limits for the week interval."""
    month: MetricsIntervalLimitTypedDict
    r"""Limits for the month interval."""
    year: MetricsIntervalLimitTypedDict
    r"""Limits for the year interval."""


class MetricsIntervalsLimits(BaseModel):
    r"""Date interval limits to get metrics for each interval."""

    hour: MetricsIntervalLimit
    r"""Limits for the hour interval."""

    day: MetricsIntervalLimit
    r"""Limits for the day interval."""

    week: MetricsIntervalLimit
    r"""Limits for the week interval."""

    month: MetricsIntervalLimit
    r"""Limits for the month interval."""

    year: MetricsIntervalLimit
    r"""Limits for the year interval."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk import utils
from polar_sdk.types import BaseModel
import pydantic
from typing import Final
from typing_extensions import Annotated


class NotPermittedType(str, Enum):
    NOT_PERMITTED = "NotPermitted"


class NotPermittedData(BaseModel):
    detail: str

    # fmt: off
    TYPE: Annotated[Final[NotPermittedType], pydantic.Field(alias="type")] = NotPermittedType.NOT_PERMITTED # type: ignore
    # fmt: on


class NotPermitted(Exception):
    r"""You don't have the permission to update this repository."""

    data: NotPermittedData

    def __init__(self, data: NotPermittedData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, NotPermittedData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .userorder import UserOrder, UserOrderTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceUserOrderTypedDict(TypedDict):
    items: List[UserOrderTypedDict]
    pagination: PaginationTypedDict


class ListResourceUserOrder(BaseModel):
    items: List[UserOrder]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class ProductPriceRecurringFixedCreateType(str, Enum):
    RECURRING = "recurring"


class ProductPriceRecurringFixedCreateAmountType(str, Enum):
    FIXED = "fixed"


class ProductPriceRecurringFixedCreateTypedDict(TypedDict):
    r"""Schema to create a recurring product price, i.e. a subscription."""

    price_amount: int
    r"""The price in cents."""
    recurring_interval: SubscriptionRecurringInterval
    r"""The recurring interval of the price."""
    type: ProductPriceRecurringFixedCreateType
    amount_type: ProductPriceRecurringFixedCreateAmountType
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""


class ProductPriceRecurringFixedCreate(BaseModel):
    r"""Schema to create a recurring product price, i.e. a subscription."""

    price_amount: int
    r"""The price in cents."""

    recurring_interval: SubscriptionRecurringInterval
    r"""The recurring interval of the price."""

    # fmt: off
    TYPE: Annotated[Final[ProductPriceRecurringFixedCreateType], pydantic.Field(alias="type")] = ProductPriceRecurringFixedCreateType.RECURRING # type: ignore
    # fmt: on

    # fmt: off
    AMOUNT_TYPE: Annotated[Final[ProductPriceRecurringFixedCreateAmountType], pydantic.Field(alias="amount_type")] = ProductPriceRecurringFixedCreateAmountType.FIXED # type: ignore
    # fmt: on

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class BenefitDiscordPropertiesInputTypedDict(TypedDict):
    r"""Properties for a benefit of type `discord`."""

    guild_id: str
    r"""The ID of the Discord server."""
    role_id: str
    r"""The ID of the Discord role to grant."""


class BenefitDiscordPropertiesInput(BaseModel):
    r"""Properties for a benefit of type `discord`."""

    guild_id: str
    r"""The ID of the Discord server."""

    role_id: str
    r"""The ID of the Discord role to grant."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .article import Article, ArticleTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceArticleTypedDict(TypedDict):
    items: List[ArticleTypedDict]
    pagination: PaginationTypedDict


class ListResourceArticle(BaseModel):
    items: List[Article]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .s3filecreatepart import S3FileCreatePart, S3FileCreatePartTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class S3FileCreateMultipartTypedDict(TypedDict):
    parts: List[S3FileCreatePartTypedDict]


class S3FileCreateMultipart(BaseModel):
    parts: List[S3FileCreatePart]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpricerecurringcustom import (
    ProductPriceRecurringCustom,
    ProductPriceRecurringCustomTypedDict,
)
from .productpricerecurringfixed import (
    ProductPriceRecurringFixed,
    ProductPriceRecurringFixedTypedDict,
)
from .productpricerecurringfree import (
    ProductPriceRecurringFree,
    ProductPriceRecurringFreeTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated


ProductPriceRecurringInputTypedDict = Union[
    ProductPriceRecurringFreeTypedDict,
    ProductPriceRecurringFixedTypedDict,
    ProductPriceRecurringCustomTypedDict,
]


ProductPriceRecurringInput = Annotated[
    Union[
        Annotated[ProductPriceRecurringCustom, Tag("custom")],
        Annotated[ProductPriceRecurringFixed, Tag("fixed")],
        Annotated[ProductPriceRecurringFree, Tag("free")],
    ],
    Discriminator(lambda m: get_discriminator(m, "amount_type", "amount_type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class RevokeTokenResponseTypedDict(TypedDict):
    pass


class RevokeTokenResponse(BaseModel):
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class LabelTypedDict(TypedDict):
    name: str
    color: str


class Label(BaseModel):
    name: str

    color: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceonetime import ProductPriceOneTime, ProductPriceOneTimeTypedDict
from .productpricerecurring_output import (
    ProductPriceRecurringOutput,
    ProductPriceRecurringOutputTypedDict,
)
from typing import Union


ProductPriceOutputTypedDict = Union[
    ProductPriceRecurringOutputTypedDict, ProductPriceOneTimeTypedDict
]


ProductPriceOutput = Union[ProductPriceRecurringOutput, ProductPriceOneTime]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitads import BenefitAds, BenefitAdsTypedDict
from .benefitadscreate import BenefitAdsCreate, BenefitAdsCreateTypedDict
from .benefitarticles import BenefitArticles, BenefitArticlesTypedDict
from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
from .benefitcustomcreate import BenefitCustomCreate, BenefitCustomCreateTypedDict
from .benefitdiscord_output import BenefitDiscordOutput, BenefitDiscordOutputTypedDict
from .benefitdiscordcreate import BenefitDiscordCreate, BenefitDiscordCreateTypedDict
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitdownloadablescreate import (
    BenefitDownloadablesCreate,
    BenefitDownloadablesCreateTypedDict,
)
from .benefitgithubrepository import (
    BenefitGitHubRepository,
    BenefitGitHubRepositoryTypedDict,
)
from .benefitgithubrepositorycreate import (
    BenefitGitHubRepositoryCreate,
    BenefitGitHubRepositoryCreateTypedDict,
)
from .benefitlicensekeys_output import (
    BenefitLicenseKeysOutput,
    BenefitLicenseKeysOutputTypedDict,
)
from .benefitlicensekeyscreate import (
    BenefitLicenseKeysCreate,
    BenefitLicenseKeysCreateTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated


BenefitsCreateBenefitCreateTypedDict = Union[
    BenefitAdsCreateTypedDict,
    BenefitDiscordCreateTypedDict,
    BenefitGitHubRepositoryCreateTypedDict,
    BenefitDownloadablesCreateTypedDict,
    BenefitLicenseKeysCreateTypedDict,
    BenefitCustomCreateTypedDict,
]


BenefitsCreateBenefitCreate = Annotated[
    Union[
        Annotated[BenefitCustomCreate, Tag("custom")],
        Annotated[BenefitAdsCreate, Tag("ads")],
        Annotated[BenefitDiscordCreate, Tag("discord")],
        Annotated[BenefitGitHubRepositoryCreate, Tag("github_repository")],
        Annotated[BenefitDownloadablesCreate, Tag("downloadables")],
        Annotated[BenefitLicenseKeysCreate, Tag("license_keys")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


BenefitsCreateResponseBenefitsCreateTypedDict = Union[
    BenefitArticlesTypedDict,
    BenefitAdsTypedDict,
    BenefitDiscordOutputTypedDict,
    BenefitGitHubRepositoryTypedDict,
    BenefitDownloadablesTypedDict,
    BenefitLicenseKeysOutputTypedDict,
    BenefitCustomTypedDict,
]
r"""Benefit created."""


BenefitsCreateResponseBenefitsCreate = Union[
    BenefitArticles,
    BenefitAds,
    BenefitDiscordOutput,
    BenefitGitHubRepository,
    BenefitDownloadables,
    BenefitLicenseKeysOutput,
    BenefitCustom,
]
r"""Benefit created."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk import utils
from polar_sdk.types import BaseModel
import pydantic
from typing import Final
from typing_extensions import Annotated


class FileNotFoundType(str, Enum):
    FILE_NOT_FOUND = "FileNotFound"


class FileNotFoundData(BaseModel):
    detail: str

    # fmt: off
    TYPE: Annotated[Final[FileNotFoundType], pydantic.Field(alias="type")] = FileNotFoundType.FILE_NOT_FOUND # type: ignore
    # fmt: on


class FileNotFound(Exception):
    data: FileNotFoundData

    def __init__(self, data: FileNotFoundData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, FileNotFoundData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .downloadablefilecreate import (
    DownloadableFileCreate,
    DownloadableFileCreateTypedDict,
)
from .organizationavatarfilecreate import (
    OrganizationAvatarFileCreate,
    OrganizationAvatarFileCreateTypedDict,
)
from .productmediafilecreate import (
    ProductMediaFileCreate,
    ProductMediaFileCreateTypedDict,
)
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated


FilesCreateFileCreateTypedDict = Union[
    DownloadableFileCreateTypedDict,
    ProductMediaFileCreateTypedDict,
    OrganizationAvatarFileCreateTypedDict,
]


FilesCreateFileCreate = Annotated[
    Union[
        Annotated[DownloadableFileCreate, Tag("downloadable")],
        Annotated[ProductMediaFileCreate, Tag("product_media")],
        Annotated[OrganizationAvatarFileCreate, Tag("organization_avatar")],
    ],
    Discriminator(lambda m: get_discriminator(m, "service", "service")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .product_input import ProductInput, ProductInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookProductCreatedPayloadType(str, Enum):
    PRODUCT_CREATED = "product.created"


class WebhookProductCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new product is created.

    **Discord & Slack support:** Basic
    """

    data: ProductInputTypedDict
    r"""A product."""
    type: WebhookProductCreatedPayloadType


class WebhookProductCreatedPayload(BaseModel):
    r"""Sent when a new product is created.

    **Discord & Slack support:** Basic
    """

    data: ProductInput
    r"""A product."""

    # fmt: off
    TYPE: Annotated[Final[WebhookProductCreatedPayloadType], pydantic.Field(alias="type")] = WebhookProductCreatedPayloadType.PRODUCT_CREATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscription_input import SubscriptionInput, SubscriptionInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookSubscriptionCreatedPayloadType(str, Enum):
    SUBSCRIPTION_CREATED = "subscription.created"


class WebhookSubscriptionCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new subscription is created.

    **Discord & Slack support:** Full
    """

    data: SubscriptionInputTypedDict
    type: WebhookSubscriptionCreatedPayloadType


class WebhookSubscriptionCreatedPayload(BaseModel):
    r"""Sent when a new subscription is created.

    **Discord & Slack support:** Full
    """

    data: SubscriptionInput

    # fmt: off
    TYPE: Annotated[Final[WebhookSubscriptionCreatedPayloadType], pydantic.Field(alias="type")] = WebhookSubscriptionCreatedPayloadType.SUBSCRIPTION_CREATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class ProductPriceOneTimeFixedCreateType(str, Enum):
    ONE_TIME = "one_time"


class ProductPriceOneTimeFixedCreateAmountType(str, Enum):
    FIXED = "fixed"


class ProductPriceOneTimeFixedCreateTypedDict(TypedDict):
    r"""Schema to create a one-time product price."""

    price_amount: int
    r"""The price in cents."""
    type: ProductPriceOneTimeFixedCreateType
    amount_type: ProductPriceOneTimeFixedCreateAmountType
    price_currency: NotRequired[str]
    r"""The currency. Currently, only `usd` is supported."""


class ProductPriceOneTimeFixedCreate(BaseModel):
    r"""Schema to create a one-time product price."""

    price_amount: int
    r"""The price in cents."""

    # fmt: off
    TYPE: Annotated[Final[ProductPriceOneTimeFixedCreateType], pydantic.Field(alias="type")] = ProductPriceOneTimeFixedCreateType.ONE_TIME # type: ignore
    # fmt: on

    # fmt: off
    AMOUNT_TYPE: Annotated[Final[ProductPriceOneTimeFixedCreateAmountType], pydantic.Field(alias="amount_type")] = ProductPriceOneTimeFixedCreateAmountType.FIXED # type: ignore
    # fmt: on

    price_currency: Optional[str] = "usd"
    r"""The currency. Currently, only `usd` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pledge_input import PledgeInput, PledgeInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookPledgeCreatedPayloadType(str, Enum):
    PLEDGE_CREATED = "pledge.created"


class WebhookPledgeCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new pledge is created. Note that this does mean that the pledge has been paid yet.

    **Discord & Slack support:** Full
    """

    data: PledgeInputTypedDict
    type: WebhookPledgeCreatedPayloadType


class WebhookPledgeCreatedPayload(BaseModel):
    r"""Sent when a new pledge is created. Note that this does mean that the pledge has been paid yet.

    **Discord & Slack support:** Full
    """

    data: PledgeInput

    # fmt: off
    TYPE: Annotated[Final[WebhookPledgeCreatedPayloadType], pydantic.Field(alias="type")] = WebhookPledgeCreatedPayloadType.PLEDGE_CREATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscription_input import SubscriptionInput, SubscriptionInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookSubscriptionUpdatedPayloadType(str, Enum):
    SUBSCRIPTION_UPDATED = "subscription.updated"


class WebhookSubscriptionUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a new subscription is updated. This event fires if the subscription is cancelled, both immediately and if the subscription is cancelled at the end of the current period.

    **Discord & Slack support:** On cancellation
    """

    data: SubscriptionInputTypedDict
    type: WebhookSubscriptionUpdatedPayloadType


class WebhookSubscriptionUpdatedPayload(BaseModel):
    r"""Sent when a new subscription is updated. This event fires if the subscription is cancelled, both immediately and if the subscription is cancelled at the end of the current period.

    **Discord & Slack support:** On cancellation
    """

    data: SubscriptionInput

    # fmt: off
    TYPE: Annotated[Final[WebhookSubscriptionUpdatedPayloadType], pydantic.Field(alias="type")] = WebhookSubscriptionUpdatedPayloadType.SUBSCRIPTION_UPDATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productonetimecreate import ProductOneTimeCreate, ProductOneTimeCreateTypedDict
from .productrecurringcreate import (
    ProductRecurringCreate,
    ProductRecurringCreateTypedDict,
)
from typing import Union


ProductsCreateProductCreateTypedDict = Union[
    ProductRecurringCreateTypedDict, ProductOneTimeCreateTypedDict
]


ProductsCreateProductCreate = Union[ProductRecurringCreate, ProductOneTimeCreate]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class OrganizationSubscribePromoteSettingsTypedDict(TypedDict):
    promote: NotRequired[bool]
    r"""Promote email subscription (free)"""
    show_count: NotRequired[bool]
    r"""Show subscription count publicly"""
    count_free: NotRequired[bool]
    r"""Include free subscribers in total count"""


class OrganizationSubscribePromoteSettings(BaseModel):
    promote: Optional[bool] = True
    r"""Promote email subscription (free)"""

    show_count: Optional[bool] = True
    r"""Show subscription count publicly"""

    count_free: Optional[bool] = True
    r"""Include free subscribers in total count"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .issue_input import IssueInput, IssueInputTypedDict
from .pledger import Pledger, PledgerTypedDict
from .pledgestate import PledgeState
from .pledgetype import PledgeType
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class PledgeInputTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    amount: int
    r"""Amount pledged towards the issue"""
    currency: str
    state: PledgeState
    r"""Current state of the pledge"""
    type: PledgeType
    r"""Type of pledge"""
    issue: IssueInputTypedDict
    r"""The issue that the pledge was made towards"""
    refunded_at: NotRequired[Nullable[datetime]]
    r"""If and when the pledge was refunded to the pledger"""
    scheduled_payout_at: NotRequired[Nullable[datetime]]
    r"""When the payout is scheduled to be made to the maintainers behind the issue. Disputes must be made before this date."""
    pledger: NotRequired[Nullable[PledgerTypedDict]]
    r"""The user or organization that made this pledge"""
    hosted_invoice_url: NotRequired[Nullable[str]]
    r"""URL of invoice for this pledge"""
    authed_can_admin_sender: NotRequired[bool]
    r"""If the currently authenticated subject can perform admin actions on behalf of the maker of the peldge"""
    authed_can_admin_received: NotRequired[bool]
    r"""If the currently authenticated subject can perform admin actions on behalf of the receiver of the peldge"""
    created_by: NotRequired[Nullable[PledgerTypedDict]]
    r"""For pledges made by an organization, or on behalf of an organization. This is the user that made the pledge. Only visible for members of said organization."""


class PledgeInput(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    amount: int
    r"""Amount pledged towards the issue"""

    currency: str

    state: PledgeState
    r"""Current state of the pledge"""

    type: PledgeType
    r"""Type of pledge"""

    issue: IssueInput
    r"""The issue that the pledge was made towards"""

    refunded_at: OptionalNullable[datetime] = UNSET
    r"""If and when the pledge was refunded to the pledger"""

    scheduled_payout_at: OptionalNullable[datetime] = UNSET
    r"""When the payout is scheduled to be made to the maintainers behind the issue. Disputes must be made before this date."""

    pledger: OptionalNullable[Pledger] = UNSET
    r"""The user or organization that made this pledge"""

    hosted_invoice_url: OptionalNullable[str] = UNSET
    r"""URL of invoice for this pledge"""

    authed_can_admin_sender: Optional[bool] = False
    r"""If the currently authenticated subject can perform admin actions on behalf of the maker of the peldge"""

    authed_can_admin_received: Optional[bool] = False
    r"""If the currently authenticated subject can perform admin actions on behalf of the receiver of the peldge"""

    created_by: OptionalNullable[Pledger] = UNSET
    r"""For pledges made by an organization, or on behalf of an organization. This is the user that made the pledge. Only visible for members of said organization."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "refunded_at",
            "scheduled_payout_at",
            "pledger",
            "hosted_invoice_url",
            "authed_can_admin_sender",
            "authed_can_admin_received",
            "created_by",
        ]
        nullable_fields = [
            "modified_at",
            "refunded_at",
            "scheduled_payout_at",
            "pledger",
            "hosted_invoice_url",
            "created_by",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .order_input import OrderInput, OrderInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookOrderCreatedPayloadType(str, Enum):
    ORDER_CREATED = "order.created"


class WebhookOrderCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new order is created.

    **Discord & Slack support:** Full
    """

    data: OrderInputTypedDict
    type: WebhookOrderCreatedPayloadType


class WebhookOrderCreatedPayload(BaseModel):
    r"""Sent when a new order is created.

    **Discord & Slack support:** Full
    """

    data: OrderInput

    # fmt: off
    TYPE: Annotated[Final[WebhookOrderCreatedPayloadType], pydantic.Field(alias="type")] = WebhookOrderCreatedPayloadType.ORDER_CREATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class PaginationTypedDict(TypedDict):
    total_count: int
    max_page: int


class Pagination(BaseModel):
    total_count: int

    max_page: int



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metricsintervalslimits import (
    MetricsIntervalsLimits,
    MetricsIntervalsLimitsTypedDict,
)
from datetime import date
from polar_sdk.types import BaseModel
from typing import TypedDict


class MetricsLimitsTypedDict(TypedDict):
    r"""Date limits to get metrics."""

    min_date: date
    r"""Minimum date to get metrics."""
    intervals: MetricsIntervalsLimitsTypedDict
    r"""Limits for each interval."""


class MetricsLimits(BaseModel):
    r"""Date limits to get metrics."""

    min_date: date
    r"""Minimum date to get metrics."""

    intervals: MetricsIntervalsLimits
    r"""Limits for each interval."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_product_ import ListResourceProduct, ListResourceProductTypedDict
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, TypedDict, Union
from typing_extensions import Annotated, NotRequired


ProductsListQueryParamOrganizationIDFilterTypedDict = Union[str, List[str]]
r"""Filter by organization ID."""


ProductsListQueryParamOrganizationIDFilter = Union[str, List[str]]
r"""Filter by organization ID."""


QueryParamBenefitIDFilterTypedDict = Union[str, List[str]]
r"""Filter products granting specific benefit."""


QueryParamBenefitIDFilter = Union[str, List[str]]
r"""Filter products granting specific benefit."""


class ProductsListRequestTypedDict(TypedDict):
    organization_id: ProductsListQueryParamOrganizationIDFilterTypedDict
    r"""Filter by organization ID."""
    is_archived: NotRequired[Nullable[bool]]
    r"""Filter on archived products."""
    is_recurring: NotRequired[Nullable[bool]]
    r"""Filter on recurring products. If `true`, only subscriptions tiers are returned. If `false`, only one-time purchase products are returned."""
    benefit_id: NotRequired[Nullable[QueryParamBenefitIDFilterTypedDict]]
    r"""Filter products granting specific benefit."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""


class ProductsListRequest(BaseModel):
    organization_id: Annotated[
        ProductsListQueryParamOrganizationIDFilter,
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]
    r"""Filter by organization ID."""

    is_archived: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter on archived products."""

    is_recurring: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter on recurring products. If `true`, only subscriptions tiers are returned. If `false`, only one-time purchase products are returned."""

    benefit_id: Annotated[
        OptionalNullable[QueryParamBenefitIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter products granting specific benefit."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["is_archived", "is_recurring", "benefit_id", "page", "limit"]
        nullable_fields = ["is_archived", "is_recurring", "benefit_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ProductsListResponseTypedDict(TypedDict):
    result: ListResourceProductTypedDict


class ProductsListResponse(BaseModel):
    next: Callable[[], Optional[ProductsListResponse]]

    result: ListResourceProduct



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class GrantType(str, Enum):
    AUTHORIZATION_CODE = "authorization_code"


class Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequestTypedDict(
    TypedDict
):
    client_id: str
    client_secret: str
    code: str
    redirect_uri: str
    grant_type: GrantType


class Onev11oauth21tokenPostXComponentsAuthorizationCodeTokenRequest(BaseModel):
    client_id: Annotated[str, FieldMetadata(form=True)]

    client_secret: Annotated[str, FieldMetadata(form=True)]

    code: Annotated[str, FieldMetadata(form=True)]

    redirect_uri: Annotated[str, FieldMetadata(form=True)]

    # fmt: off
    GRANT_TYPE: Annotated[Final[GrantType], pydantic.Field(alias="grant_type"), FieldMetadata(form=True)] = GrantType.AUTHORIZATION_CODE # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType(str, Enum):
    REFRESH_TOKEN = "refresh_token"


class Onev11oauth21tokenPostXComponentsRefreshTokenRequestTypedDict(TypedDict):
    client_id: str
    client_secret: str
    refresh_token: str
    grant_type: Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType


class Onev11oauth21tokenPostXComponentsRefreshTokenRequest(BaseModel):
    client_id: Annotated[str, FieldMetadata(form=True)]

    client_secret: Annotated[str, FieldMetadata(form=True)]

    refresh_token: Annotated[str, FieldMetadata(form=True)]

    # fmt: off
    GRANT_TYPE: Annotated[Final[Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType], pydantic.Field(alias="grant_type"), FieldMetadata(form=True)] = Onev11oauth21tokenPostXComponentsRefreshTokenRequestGrantType.REFRESH_TOKEN # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .product_output import ProductOutput, ProductOutputTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceProductTypedDict(TypedDict):
    items: List[ProductOutputTypedDict]
    pagination: PaginationTypedDict


class ListResourceProduct(BaseModel):
    items: List[ProductOutput]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organization_input import OrganizationInput, OrganizationInputTypedDict
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class WebhookOrganizationUpdatedPayloadType(str, Enum):
    ORGANIZATION_UPDATED = "organization.updated"


class WebhookOrganizationUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when a organization is updated.

    **Discord & Slack support:** Basic
    """

    data: OrganizationInputTypedDict
    type: WebhookOrganizationUpdatedPayloadType


class WebhookOrganizationUpdatedPayload(BaseModel):
    r"""Sent when a organization is updated.

    **Discord & Slack support:** Basic
    """

    data: OrganizationInput

    # fmt: off
    TYPE: Annotated[Final[WebhookOrganizationUpdatedPayloadType], pydantic.Field(alias="type")] = WebhookOrganizationUpdatedPayloadType.ORGANIZATION_UPDATED # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ProductBenefitsUpdateTypedDict(TypedDict):
    r"""Schema to update the benefits granted by a product."""

    benefits: List[str]
    r"""List of benefit IDs. Each one must be on the same organization as the product."""


class ProductBenefitsUpdate(BaseModel):
    r"""Schema to update the benefits granted by a product."""

    benefits: List[str]
    r"""List of benefit IDs. Each one must be on the same organization as the product."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitads import BenefitAds, BenefitAdsTypedDict
from .benefitarticles import BenefitArticles, BenefitArticlesTypedDict
from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
from .benefitdiscord_output import BenefitDiscordOutput, BenefitDiscordOutputTypedDict
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitgithubrepository import (
    BenefitGitHubRepository,
    BenefitGitHubRepositoryTypedDict,
)
from .benefitlicensekeys_output import (
    BenefitLicenseKeysOutput,
    BenefitLicenseKeysOutputTypedDict,
)
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict, Union


BenefitTypedDict = Union[
    BenefitArticlesTypedDict,
    BenefitAdsTypedDict,
    BenefitDiscordOutputTypedDict,
    BenefitGitHubRepositoryTypedDict,
    BenefitDownloadablesTypedDict,
    BenefitLicenseKeysOutputTypedDict,
    BenefitCustomTypedDict,
]


Benefit = Union[
    BenefitArticles,
    BenefitAds,
    BenefitDiscordOutput,
    BenefitGitHubRepository,
    BenefitDownloadables,
    BenefitLicenseKeysOutput,
    BenefitCustom,
]


class ListResourceBenefitTypedDict(TypedDict):
    items: List[BenefitTypedDict]
    pagination: PaginationTypedDict


class ListResourceBenefit(BaseModel):
    items: List[Benefit]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .s3fileuploadcompletedpart import (
    S3FileUploadCompletedPart,
    S3FileUploadCompletedPartTypedDict,
)
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class FileUploadCompletedTypedDict(TypedDict):
    id: str
    path: str
    parts: List[S3FileUploadCompletedPartTypedDict]


class FileUploadCompleted(BaseModel):
    id: str

    path: str

    parts: List[S3FileUploadCompletedPart]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .order_output import OrderOutput, OrderOutputTypedDict
from .pagination import Pagination, PaginationTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceOrderTypedDict(TypedDict):
    items: List[OrderOutputTypedDict]
    pagination: PaginationTypedDict


class ListResourceOrder(BaseModel):
    items: List[OrderOutput]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationfeaturesettings import (
    OrganizationFeatureSettings,
    OrganizationFeatureSettingsTypedDict,
)
from .organizationprofilesettings import (
    OrganizationProfileSettings,
    OrganizationProfileSettingsTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional, TypedDict
from typing_extensions import NotRequired


class OrganizationUpdateTypedDict(TypedDict):
    name: NotRequired[Nullable[str]]
    avatar_url: NotRequired[Nullable[str]]
    default_upfront_split_to_contributors: NotRequired[Nullable[int]]
    pledge_badge_show_amount: NotRequired[bool]
    billing_email: NotRequired[Nullable[str]]
    default_badge_custom_content: NotRequired[Nullable[str]]
    pledge_minimum_amount: NotRequired[int]
    total_monthly_spending_limit: NotRequired[Nullable[int]]
    per_user_monthly_spending_limit: NotRequired[Nullable[int]]
    donations_enabled: NotRequired[bool]
    profile_settings: NotRequired[Nullable[OrganizationProfileSettingsTypedDict]]
    feature_settings: NotRequired[Nullable[OrganizationFeatureSettingsTypedDict]]


class OrganizationUpdate(BaseModel):
    name: OptionalNullable[str] = UNSET

    avatar_url: OptionalNullable[str] = UNSET

    default_upfront_split_to_contributors: OptionalNullable[int] = UNSET

    pledge_badge_show_amount: Optional[bool] = False

    billing_email: OptionalNullable[str] = UNSET

    default_badge_custom_content: OptionalNullable[str] = UNSET

    pledge_minimum_amount: Optional[int] = 2000

    total_monthly_spending_limit: OptionalNullable[int] = UNSET

    per_user_monthly_spending_limit: OptionalNullable[int] = UNSET

    donations_enabled: Optional[bool] = False

    profile_settings: OptionalNullable[OrganizationProfileSettings] = UNSET

    feature_settings: OptionalNullable[OrganizationFeatureSettings] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "avatar_url",
            "default_upfront_split_to_contributors",
            "pledge_badge_show_amount",
            "billing_email",
            "default_badge_custom_content",
            "pledge_minimum_amount",
            "total_monthly_spending_limit",
            "per_user_monthly_spending_limit",
            "donations_enabled",
            "profile_settings",
            "feature_settings",
        ]
        nullable_fields = [
            "name",
            "avatar_url",
            "default_upfront_split_to_contributors",
            "billing_email",
            "default_badge_custom_content",
            "total_monthly_spending_limit",
            "per_user_monthly_spending_limit",
            "profile_settings",
            "feature_settings",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class BenefitGrantLicenseKeysPropertiesTypedDict(TypedDict):
    license_key_id: str
    display_key: str


class BenefitGrantLicenseKeysProperties(BaseModel):
    license_key_id: str

    display_key: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import TypedDict


class UserSubscriptionUpdateTypedDict(TypedDict):
    product_price_id: str


class UserSubscriptionUpdate(BaseModel):
    product_price_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .userinfoorganization import UserInfoOrganization, UserInfoOrganizationTypedDict
from .userinfouser import UserInfoUser, UserInfoUserTypedDict
from typing import Union


Oauth2UserinfoResponseOauth2UserinfoTypedDict = Union[
    UserInfoOrganizationTypedDict, UserInfoUserTypedDict
]
r"""Successful Response"""


Oauth2UserinfoResponseOauth2Userinfo = Union[UserInfoOrganization, UserInfoUser]
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .usersubscriptionupdate import (
    UserSubscriptionUpdate,
    UserSubscriptionUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import TypedDict
from typing_extensions import Annotated


class UsersSubscriptionsUpdateRequestTypedDict(TypedDict):
    id: str
    r"""The subscription ID."""
    user_subscription_update: UserSubscriptionUpdateTypedDict


class UsersSubscriptionsUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The subscription ID."""

    user_subscription_update: Annotated[
        UserSubscriptionUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .repository_output import RepositoryOutput, RepositoryOutputTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceRepositoryTypedDict(TypedDict):
    items: List[RepositoryOutputTypedDict]
    pagination: PaginationTypedDict


class ListResourceRepository(BaseModel):
    items: List[RepositoryOutput]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceonetime import ProductPriceOneTime, ProductPriceOneTimeTypedDict
from .productpricerecurring_input import (
    ProductPriceRecurringInput,
    ProductPriceRecurringInputTypedDict,
)
from typing import Union


ProductPriceInputTypedDict = Union[
    ProductPriceRecurringInputTypedDict, ProductPriceOneTimeTypedDict
]


ProductPriceInput = Union[ProductPriceRecurringInput, ProductPriceOneTime]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .usersubscription import UserSubscription, UserSubscriptionTypedDict
from polar_sdk.types import BaseModel
from typing import List, TypedDict


class ListResourceUserSubscriptionTypedDict(TypedDict):
    items: List[UserSubscriptionTypedDict]
    pagination: PaginationTypedDict


class ListResourceUserSubscription(BaseModel):
    items: List[UserSubscription]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionrecurringinterval import SubscriptionRecurringInterval
from enum import Enum
from polar_sdk.types import BaseModel
import pydantic
from typing import Final, TypedDict
from typing_extensions import Annotated


class ProductPriceRecurringFreeCreateType(str, Enum):
    RECURRING = "recurring"


class ProductPriceRecurringFreeCreateAmountType(str, Enum):
    FREE = "free"


class ProductPriceRecurringFreeCreateTypedDict(TypedDict):
    r"""Schema to create a free recurring product price, i.e. a subscription."""

    recurring_interval: SubscriptionRecurringInterval
    r"""The recurring interval of the price."""
    type: ProductPriceRecurringFreeCreateType
    amount_type: ProductPriceRecurringFreeCreateAmountType


class ProductPriceRecurringFreeCreate(BaseModel):
    r"""Schema to create a free recurring product price, i.e. a subscription."""

    recurring_interval: SubscriptionRecurringInterval
    r"""The recurring interval of the price."""

    # fmt: off
    TYPE: Annotated[Final[ProductPriceRecurringFreeCreateType], pydantic.Field(alias="type")] = ProductPriceRecurringFreeCreateType.RECURRING # type: ignore
    # fmt: on

    # fmt: off
    AMOUNT_TYPE: Annotated[Final[ProductPriceRecurringFreeCreateAmountType], pydantic.Field(alias="amount_type")] = ProductPriceRecurringFreeCreateAmountType.FREE # type: ignore
    # fmt: on



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filedownload import FileDownload, FileDownloadTypedDict
from polar_sdk.types import BaseModel
from typing import TypedDict


class DownloadableReadTypedDict(TypedDict):
    id: str
    benefit_id: str
    file: FileDownloadTypedDict


class DownloadableRead(BaseModel):
    id: str

    benefit_id: str

    file: FileDownload



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class UserBenefitSortProperty(str, Enum):
    GRANTED_AT = "granted_at"
    MINUS_GRANTED_AT = "-granted_at"
    TYPE = "type"
    MINUS_TYPE = "-type"
    ORGANIZATION = "organization"
    MINUS_ORGANIZATION = "-organization"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from polar_sdk import utils
from polar_sdk.types import BaseModel
import pydantic
from typing import Final
from typing_extensions import Annotated


class AlreadyCanceledSubscriptionType(str, Enum):
    ALREADY_CANCELED_SUBSCRIPTION = "AlreadyCanceledSubscription"


class AlreadyCanceledSubscriptionData(BaseModel):
    detail: str

    # fmt: off
    TYPE: Annotated[Final[AlreadyCanceledSubscriptionType], pydantic.Field(alias="type")] = AlreadyCanceledSubscriptionType.ALREADY_CANCELED_SUBSCRIPTION # type: ignore
    # fmt: on


class AlreadyCanceledSubscription(Exception):
    data: AlreadyCanceledSubscriptionData

    def __init__(self, data: AlreadyCanceledSubscriptionData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, AlreadyCanceledSubscriptionData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceonetimecustom import (
    ProductPriceOneTimeCustom,
    ProductPriceOneTimeCustomTypedDict,
)
from .productpriceonetimefixed import (
    ProductPriceOneTimeFixed,
    ProductPriceOneTimeFixedTypedDict,
)
from .productpriceonetimefree import (
    ProductPriceOneTimeFree,
    ProductPriceOneTimeFreeTypedDict,
)
from typing import Union


ProductPriceOneTimeTypedDict = Union[
    ProductPriceOneTimeFreeTypedDict,
    ProductPriceOneTimeFixedTypedDict,
    ProductPriceOneTimeCustomTypedDict,
]


ProductPriceOneTime = Union[
    ProductPriceOneTimeFree, ProductPriceOneTimeFixed, ProductPriceOneTimeCustom
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitads import BenefitAds, BenefitAdsTypedDict
from .benefitadsupdate import BenefitAdsUpdate, BenefitAdsUpdateTypedDict
from .benefitarticles import BenefitArticles, BenefitArticlesTypedDict
from .benefitarticlesupdate import BenefitArticlesUpdate, BenefitArticlesUpdateTypedDict
from .benefitcustom import BenefitCustom, BenefitCustomTypedDict
from .benefitcustomupdate import BenefitCustomUpdate, BenefitCustomUpdateTypedDict
from .benefitdiscord_output import BenefitDiscordOutput, BenefitDiscordOutputTypedDict
from .benefitdiscordupdate import BenefitDiscordUpdate, BenefitDiscordUpdateTypedDict
from .benefitdownloadables import BenefitDownloadables, BenefitDownloadablesTypedDict
from .benefitdownloadablesupdate import (
    BenefitDownloadablesUpdate,
    BenefitDownloadablesUpdateTypedDict,
)
from .benefitgithubrepository import (
    BenefitGitHubRepository,
    BenefitGitHubRepositoryTypedDict,
)
from .benefitgithubrepositoryupdate import (
    BenefitGitHubRepositoryUpdate,
    BenefitGitHubRepositoryUpdateTypedDict,
)
from .benefitlicensekeys_output import (
    BenefitLicenseKeysOutput,
    BenefitLicenseKeysOutputTypedDict,
)
from .benefitlicensekeysupdate import (
    BenefitLicenseKeysUpdate,
    BenefitLicenseKeysUpdateTypedDict,
)
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import TypedDict, Union
from typing_extensions import Annotated


BenefitsUpdateBenefitUpdateTypedDict = Union[
    BenefitArticlesUpdateTypedDict,
    BenefitAdsUpdateTypedDict,
    BenefitCustomUpdateTypedDict,
    BenefitDiscordUpdateTypedDict,
    BenefitGitHubRepositoryUpdateTypedDict,
    BenefitDownloadablesUpdateTypedDict,
    BenefitLicenseKeysUpdateTypedDict,
]


BenefitsUpdateBenefitUpdate = Union[
    BenefitArticlesUpdate,
    BenefitAdsUpdate,
    BenefitCustomUpdate,
    BenefitDiscordUpdate,
    BenefitGitHubRepositoryUpdate,
    BenefitDownloadablesUpdate,
    BenefitLicenseKeysUpdate,
]


class BenefitsUpdateRequestTypedDict(TypedDict):
    id: str
    request_body: BenefitsUpdateBenefitUpdateTypedDict


class BenefitsUpdateRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    request_body: Annotated[
        BenefitsUpdateBenefitUpdate,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


BenefitsUpdateResponseBenefitsUpdateTypedDict = Union[
    BenefitArticlesTypedDict,
    BenefitAdsTypedDict,
    BenefitDiscordOutputTypedDict,
    BenefitGitHubRepositoryTypedDict,
    BenefitDownloadablesTypedDict,
    BenefitLicenseKeysOutputTypedDict,
    BenefitCustomTypedDict,
]
r"""Benefit updated."""


BenefitsUpdateResponseBenefitsUpdate = Union[
    BenefitArticles,
    BenefitAds,
    BenefitDiscordOutput,
    BenefitGitHubRepository,
    BenefitDownloadables,
    BenefitLicenseKeysOutput,
    BenefitCustom,
]
r"""Benefit updated."""
